# defaults
set ns_tcp(maxburst) 0
set ns_tcp(maxcwnd) 0
set ns_tcp(window) 20
set ns_tcp(window-init) 1
set ns_tcp(overhead) 0
set ns_tcp(ecn) 0
set ns_tcp(packet-size) 1000
set ns_tcp(bug-fix) true
set ns_tcp(tcp-tick) 0.1
set ns_tcpnewreno(changes) 0
# vegas thresholds
set ns_tcp(v-alpha) 1	
set ns_tcp(v-beta)  3
set ns_tcp(v-gamma) 1
# trace
set ns_trace(show_tcphdr) false
# full tcp params; full-tcp is new and experimental
set ns_fulltcp(segsperack) 1
set ns_fulltcp(segsize) 536
set ns_fulltcp(tcprexmtthresh) 3
set ns_fulltcp(nodelay) false
set ns_fulltcp(data_on_syn) false
set ns_fulltcp(iss) 0
set ns_fulltcp(dupseg_fix) true
set ns_fulltcp(dupack_reset) false
#
# These parameters have all been verified: window, window-init,
#   packet-size, bug-fix (tcp, tcp-reno), overhead (tcp but not tcp-reno), 
#   ecn (tcp, tcp-reno, tcp-sink, and tcp-sink-da).  
#   The "overhead" option has not been implemented for tcp-reno.
# The following parameters are for obscure options probably only of interest 
#  to Sally, and have not been tested in the new simulator: window-option, 
#  window-constant, window-thresh.
set ns_tcp(window-option) 1
set ns_tcp(window-constant) 4
set ns_tcp(window-thresh) 0.002

# These parameters have all been verified: thresh, maxthresh, q_weight,
#  linterm, setbit, bytes.
# These parameters have not been tested: mean_pktsize, wait, drop-tail.
set ns_red(bytes) false
set ns_red(thresh) 5
set ns_red(maxthresh) 15
set ns_red(mean_pktsize) 500
set ns_red(q_weight) 0.002
set ns_red(wait) true
set ns_red(linterm) 10
set ns_red(setbit) false
set ns_red(drop-tail) false
# concise, time_diff, ave_diff:  for concise statistics
set ns_red(concise) false	
set ns_red(time_diff) 0.1	
set ns_red(ave_diff) 0.2	
# These parameters are for obscure options probably only of interest to
#  Sally, and have not been tested in the new simulator: doubleq, dqthresh,
#  subclasses.
set ns_red(doubleq) false	
set ns_red(dqthresh) 50		
set ns_red(subclasses) 1	
# These parameters below are only used for RED queues with RED subclasses.
# For Sally, for experimental purposes only.
set ns_red(thresh1) 5
set ns_red(maxthresh1) 15
set ns_red(mean_pktsize1) 500

set ns_cbq(algorithm) 0
set ns_cbq(max-pktsize) 1024
set ns_class(priority) 0
set ns_class(depth) 0
set ns_class(allotment) 0.0
set ns_class(maxidle) 4ms
set ns_class(minidle) -1000ms
set ns_class(extradelay) 0
set ns_class(plot) false
# set ns_class(qdisc) red
set ns_class(qdisc) drop-tail
# These parameters have all been verified: priority, depth, allotment,
#  maxidle, extradelay.

set ns_sink(packet-size) 40
set ns_delsink(interval) 100ms
set ns_sacksink(max-sack-blocks) 3
set ns_cbr(interval) 3.75ms
set ns_cbr(random) 0
set ns_cbr(packet-size) 210
set ns_rlm(interval) 3.75ms
set ns_rlm(packet-size) 210

set ns_ivs(S) 1
set ns_ivs(R) 1
set ns_ivs(state) 0
set ns_ivs(rttShift) 0
set ns_ivs(keyShift) 0
set ns_ivs(key) 0
set ns_ivs(maxrtt) 0.0
set ns_ivs(ignoreR) 0

set ns_source(maxpkts) 0
set ns_telnet(interval) 1000ms
set ns_bursty(interval) 0
set ns_bursty(burst-size) 2
set ns_message(packet-size) 40

set ns_facktcp(ss-div4) 1
set ns_facktcp(rampdown) 1

set ns_link(state) 1                  ;# set state of link up
set ns_link(bandwidth) 1.5Mb
set ns_link(delay) 100ms
set ns_link(queue-limit) 50
set ns_link(link-concise) 0
set ns_link(queue-in-bytes) false
set ns_link(mean_pktsize) 1024
set ns_lossy_uniform(loss_prob) 0.00
set ns_lossy_patt(loss_prob) 0.00
set ns_lossy_patt(loss_burst) 1
set ns_lossy_patt(loss_interval) 0
# These packets have all been verified: link(bandwidth), link(delay),
#  link(queue-limit), telnet(interval), delsink(interval).

set ns_queue(size) 50
#  Is queue(size) used anywhere?

# The SRM params
set ns_srm(repair-size) 210
set ns_srm(request-size) 48
set ns_srm(request-delay) 50ms
set ns_srm(repair-delay) 50ms
set ns_srm(ignore-delay) 50ms
set ns_srm(request-c1) 0.5
set ns_srm(request-c2) 2.0
set ns_srm(repair-d1) 0.5
set ns_srm(repair-d2) 2.0
set ns_srm(ignore-constant) 3.0
set ns_srm(backoff) 2.0

proc ns_connect { src sink } {
	$src connect [$sink addr] [$sink port]
	$sink connect [$src addr] [$src port]
}

proc ns_duplex { n1 n2 bw delay type } {
	set link0 [ns link $n1 $n2 $type]
	$link0 set bandwidth $bw
	$link0 set delay $delay
	set link1 [ns link $n2 $n1 $type]
	$link1 set bandwidth $bw
	$link1 set delay $delay
	return "$link0 $link1"
}

#
# Create a source/sink connection pair and return the source agent.
# 
proc ns_create_connection { srcType srcNode sinkType sinkNode class } {
	set src [ns agent $srcType $srcNode]
	set sink [ns agent $sinkType $sinkNode]
	$src set class $class
	$sink set class $class
	if { $srcType == "tcp-full" } {
		$src connect [$sink addr] [$sink port]
		$sink listen    ; # will figure out who its peer is
	} else {
		ns_connect $src $sink
	}
	return $src
}

#
# Create a Reno TCP connection with the source/sink pair.  
#   Maximum window $window, start time $start, class $class.
#
proc ns_create_reno { tcpSrc tcpDst window start class} {
        set tcp [ns_create_connection tcp-reno $tcpSrc tcp-sink $tcpDst $class]
        $tcp set window $window
        set ftp [$tcp source ftp]
        ns at $start "$ftp start"
	return $tcp
}

#
# Create a source/sink connection pair for a CBR connection, and return
#   the source agent.
#
proc ns_create_cbr { srcNode sinkNode pktSize interval class } {
	set src [ns agent cbr $srcNode]
	set sink [ns agent loss-monitor $sinkNode]
        $src set interval $interval
        $src set packet-size $pktSize
        $src set class $class
        ns_connect $src $sink
	return $src
}

#
# Create a source/sink connection pair for a CBR connection, and return
#   the source agent.
#
proc ns_create_cbr2 { srcNode sinkNode pktSize interval class } {
	set src [ns agent cbr $srcNode]
	set sink [ns agent loss-monitor $sinkNode]
        $src set interval $interval
        $src set packet-size $pktSize
        $src set class $class
        ns_connect $src $sink
	return "$src $sink"
}

#
# Procedure for backward compatibility with old ns commands.
# Preferred approach is to allocate objects with the "new" command.
#
proc ns { cmd args } {
	global ns_compat
	if { $cmd == "node" } {
		if ![info exists ns_compat(route-logic)] {
			set ns_compat(route-logic) [new route-logic]
		}
		set node [new node]
		$ns_compat(route-logic) insert $node
		lappend ns_compat(nodes-list) $node
		return $node
	}
	if { $cmd == "link" } {
		if { [llength $args] == 3 } {
			set src [lindex $args 0]
			set dst [lindex $args 1]
			set type [lindex $args 2]
			set L [new link $type]
			$L install $src $dst
			set ns_compat(link:$src:$dst) $L
			return $L
		}
		if { [llength $args] == 2 } {
			set src [lindex $args 0]
			set dst [lindex $args 1]
			return $ns_compat(link:$src:$dst)
		}
		if { $args == "" } {
			set L ""
			foreach v [array names ns_compat] {
				if [string match link:* $v] {
					lappend L $ns_compat($v)
				}
			}
			return $L
		}
	}
	if { $cmd == "link?" } {
	    if { [llength $args] != 1 } {
		return ""
	    }
	    set L ""
	    set node [lindex $args 0]
	    foreach v [array names ns_compat "link:$node:*"] {
		set idx [expr [string last ":" $v] + 1]
		lappend L [string range $v $idx end]
	    }
	    return $L
	}
	if { $cmd == "agent" } {
		if { [llength $args] == 2 } {
			set type [lindex $args 0]
			set node [lindex $args 1]
			set agent [new agent $type]
			$agent node $node
			return $agent
		}
	}
	if { $cmd == "route-object" } {
	    set node [lindex $args 0]
	    if [info exists ns_compat(rtobj:$node)] {
		return $ns_compat(rtobj:$node)
	    }
	    set rtobj [ns agent rtobject $node]
	    $rtobj init
	    set ns_compat(rtobj:$node) $rtobj
	    set ns_compat(rtobj:init) 1
	    return $rtobj
	}
	if { $cmd == "route-object?" } {
	    set node [lindex $args 0]
	    return $ns_compat(rtobj:$node)
	}
	if { $cmd == "route-proto?" } {
	    set node [lindex $args 0]
	    if { [llength $args] == 1 } {
		# return all proto for that node
		set pr ""
		foreach pidx [array names ns_compat "rtproto:$node:*"] {
		    set idx [expr [string last ":" $pidx] + 1]
		    lappend pr [string range $pidx $idx end]
		}
		return $pr
	    }
	    if { [llength $args] == 2 } {
		# special case
		set proto [lindex $args 1]
		if [info exists ns_compat(rtproto:$node:$proto)] {
		    return $ns_compat(rtproto:$node:$proto)
		} else {
		    return ""
		}
	    }
	    set pr ""
	    foreach proto [lrange $args 1 end] {
		if [info exists ns_compat(rtproto:$node:$proto)] {
		    lappend pr $ns_compat(rtproto:$node:$proto)
		} else {
		    lappend pr ""
		}
	    }
	    return $pr
	}
	if { $cmd == "route-proto" } {
	    if { [llength $args] == 1 } {
		return ns route-proto? $args
	    }
	    set node [lindex $args 0]
	    set rtobj [ns route-object $node]
	    set pr ""
	    foreach proto [lrange $args 1 end] {
		set rtproto [ns agent rtproto $node]
		$rtproto proto $proto
		$rtproto rtagent $rtobj
		set ns_compat(rtproto:$node:$proto) $rtproto
		lappend pr $rtproto
	    }
	    return pr
	}
	if { $cmd == "trace" } {
		set trace [new trace]
		return $trace
	}
	if { $cmd == "at" } {
		eval ns-at $args
		return
	}
	if { $cmd == "now" } {
		return [ns-now]
	}
	if { $cmd == "random" } {
		return [eval ns-random $args]
	}
	if { $cmd == "run" } {
#		$ns_compat(route-logic) compute-routes
		eval ns_init_routing $args
		ns-run
		return
	}
	puts stderr "ns: backward compat doesn't handle 'ns $cmd $args'"
	exit 1
}

#
# XXX hack: the Tcl class calls "tkerror" when it hits an
# error evaluating a tcl stub (e.g., from an ns-at command).
# Dump a stack trace and exit.
# 
proc tkerror s {
	global errorInfo
	puts stderr "ns: tcl eval error"
	puts stderr $errorInfo
	exit 1
}

##########################################################################
# CBQ									 #
##########################################################################

#
# Create a CBQ (class-based queueing) class with the specified parameters.
#	use ns_class(qdisc) as the default
#
proc ns_create_class { parent borrow allot maxIdle notused priority depth extraDelay} { 
	global ns_class ns_link
        set class [new class]
        set class1 [ns_class_params $class $parent $borrow $allot $maxIdle \
	 0 $priority $depth $extraDelay 0]
	set qdisc [new link $ns_class(qdisc)]
	$class1 qdisc $qdisc
        return $class1 
}

# "Mbps" gives the link bandwidth in Mbps.
proc ns_create_class1 { parent borrow allot maxIdle notused priority depth extraDelay Mbps} {
	global ns_class ns_link
        set class [new class]
        set class1 [ns_class_params $class $parent $borrow $allot $maxIdle \
           0 $priority $depth $extraDelay $Mbps]
	set qdisc [new link $ns_class(qdisc)]
	$class1 qdisc $qdisc
        return $class1
}  

#
# Assign the following parameters to the CBQ class.
# "Minidle" is no longer used. 
#
proc ns_class_params { class parent borrow allot maxIdle notused priority depth extraDelay Mbps} {  
        $class parent $parent
        $class borrow $borrow
        $class set allotment $allot
        $class set maxidle $maxIdle
        $class set priority $priority
        $class set depth $depth
        $class set extradelay $extraDelay
        set class1 [ns_class_maxIdle $class $allot $maxIdle $priority $Mbps]
        return $class1  
}

#
# If $maxIdle is "auto", set maxIdle to Max[t(1/p-1)(1-g^n)/g^n, t(1-g)].
# For p = allotment, t = packet transmission time, g = weight for EWMA.
# The parameter t is calculated for a medium-sized 1000-byte packet.
#
proc ns_class_maxIdle { class allot maxIdle priority Mbps } {
        if { $maxIdle == "auto" } {
		set g 0.9375
		set n [expr 8 * $priority]
		set gTOn [expr pow($g, $n)]
		set first [expr ((1/$allot) - 1) * (1-$gTOn) / $gTOn ]
		set second [expr (1 - $g)]
		set packetsize 1000
		set t [expr ($packetsize * 8)/($Mbps * 1000000) ]
		if { $first > $second } {
		  	$class set maxidle [expr $t * $first]
		} else {
			$class set maxidle [expr $t * $second]
		}
	} else {
		$class set maxidle $maxIdle
	}
	return $class
}

#
# We need to recreate some of the C++ random functions here in Tcl
#

proc ns_rand_uniform {a b} {
    return [expr $a + ( ( $b - $a ) * ( [ns random] * 1.0 / 0x7fffffff ) )]
}

proc ns_rand_exponential {} {
    return [expr - log([ns_rand_uniform 0 1])]
}

proc ns_rand_integer k {
    return [expr [ns random] % abs($k)]
}

#
# Route event queues
#
proc ns_rtq_enq { time func src dst args } {
    global ns_rtq

    if { ! [ info exists ns_rtq($time) ] } {
	ns at $time "ns_rtq_deq $time"
    }
    lappend ns_rtq($time) "$func $src $dst $args"
}

proc ns_rtq_deq time {
    global ns_rtq
    global ns_compat

    if { ! [ info exists ns_rtq($time) ] } {
	puts stderr "ns: events in queue are [array get ns_rtq]"
	puts stderr "ns: cannot find event at time [ns now] ( = $time)"
    }
    foreach event $ns_rtq($time) {
	foreach node [lrange $event 1 2] {
	    set wake_up($node) 1
	}
	eval $event			;# event == function src dst args
    }
    foreach node [array names wake_up] {
	$node notify-ifchanged
	$ns_compat(rtobj:$node) notify-ifchanged
    }
    unset ns_rtq($time)
}

#
# Route Models
#
# Route configuration parameters
set ns_rtconf(pref-dv)		120

proc ns_init_routing { { rt_type "static" } args } {
    global ns_compat

    if { [ info exists ns_rt_${rt_type} ] } {
	puts stderr "ns: invalid route model $rt_type specified, ignored"
	set rt_type "static"
    }
    eval ns_rt_${rt_type} $args
    set ns_compat(route-model) $rt_type
}

proc ns_init_routeobjects {} {
    global ns_compat

    if { ! [info exists ns_compat(rtobj:init)] } { 
	foreach node $ns_compat(nodes-list) {
	    ns route-proto $node "DV"
	}
    }
    foreach node $ns_compat(nodes-list) {
	foreach nbr [ns link? $node] {
	    foreach proto [ns route-proto? $node] {
		set npr [ns route-proto? $nbr $proto]
		if { [string compare $npr ""] != 0 } {
		    set lpr [ns link $node $nbr]
		    [ns route-proto? $node $proto] peer-init $lpr $npr
		}
	    }
	}
    }
    return
}

proc ns_rt_ args {
    if { ![info exists ns_compat(route-model)] ||
	 $ns_compat(route-model) == "static" } {
        ns_rt_static $args
    }
}

proc ns_rt_static { args } {
    global ns_compat

    $ns_compat(route-logic) compute-routes
}

proc ns_rt_trace { args } {
    global ns_compat

    set ns_compat(trace-file) [lindex $args 0]
    puts stderr "ns: trace based models are not implemented yet"
    exit 1
}

set ns_rtconf(exp_up_intvl_)	10.0
set ns_rtconf(exp_down_intvl_)	1.0

proc ns_rt_exponential args {
    global ns_rtconf

    ns_init_routeobjects
    set src [lindex $args 0]
    set dst [lindex $args 1]
    if { [llength $args] >= 3 } {
	set ns_rtconf(exp_up_intvl) [lindex $args 2]
    } else {
	set ns_rtconf(exp_up_intvl) $ns_rtconf(exp_up_intvl_)
    }
    if { [llength $args] >= 4 } {
	set ns_rtconf(exp_down_intvl) [lindex $args 3]
    } else {
	set ns_rtconf(exp_down_intvl) $ns_rtconf(exp_down_intvl_)
    }

    set fev [expr 0.5 + [ns_rand_exponential] * $ns_rtconf(exp_down_intvl)]
    ns_rtq_enq $fev "ns_rtm_exponential" $src $dst "down"
#    ns_rtq_enq 0.5 "ns_rtm_exponential" $src $dst "up"
}

proc ns_rtm_exponential { src dst op } {
    global ns_rtconf

    set tnow [ns now]
    puts stderr [format "ns: link <%s_-%s_> $op at %7.4g" $src $dst $tnow]
    [ns link $src $dst] $op
    [ns link $dst $src] $op
    if { $op == "up" } {
	set nextop "down"
    } else {
	set nextop "up"
    }
    set nev [expr $tnow + [ns_rand_exponential] * $ns_rtconf(exp_${op}_intvl)]
    ns_rtq_enq $nev "ns_rtm_exponential" $src $dst $nextop
}

