/*                                                                           */
/*      Copyright (c) 1997 by the University of Southern California          */
/*      All rights reserved.                                                 */
/*                                                                           */
/*      Permission to use, copy, modify, and distribute this software and    */
/*      its documentation in source and binary forms for any purpose and     */
/*      without fee is hereby granted, provided that both the above          */
/*      copyright notice and this permission notice appear in all copies,    */
/*      and that any documentation, advertising materials, and other         */
/*      materials related to such distribution and use acknowledge that      */
/*      the software was developed in part by the University of Southern     */
/*      California, Information Sciences Institute.  The name of the         */
/*      University may not be used to endorse or promote products derived    */
/*      from this software without specific prior written permission.        */
/*                                                                           */
/*                                                                           */
/*      THE UNIVERSITY OF SOUTHERN CALIFORNIA makes no representations about */
/*      the suitability of this software for any purpose.  THIS SOFTWARE IS  */
/*      PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,      */
/*      INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF             */
/*      MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.                */
/*                                                                           */

/*
 * @(#) $Header: rtglib.h,v 1.2 97/01/23 23:26:37 kfall Exp $ (USC/ISI)
 */

// requires "Tcl.h"
// requires "object.h"
// requires "scheduler.h"
// requires "packet.h'
// requires "agent.h"
// requires "node.h"

#ifndef ns_rtglib_h
#define ns_rtglib_h

#include <sys/types.h>
#include "agent.h"

class	Link;
class	Node;


class	rtAgent;
class	rtProto;
struct	rtPeer;

#define	MINCOST		0

#define MINPREF		0
#define MAXPREF		127
#define INVPREF		-1

struct RIB {
	Link*	if_;			// outbound InterFace
	rtPeer*	peer_;			// peer this route was learned from
	rtProto* proto_;		// proto this route was learned from
	char	cost_;
	char	pref_;
};

struct Link_State {			// LinkState has been usurped to
	Link*	link_;			// store multicast state per link
	int	state_;
	Link_State*	next_;
};

class rtObject : public Agent {
	Link_State*	linkstate_;
	rtProto*	rtproto_;
	RIB*		rts_;		// aka LocRIB
	int		proto_rts_changed_;
	double		time_up_,	// of last install
		time_dn_;		// of last uninstall
	inline int	check_link_states();
	inline int	BetterRoute(const RIB *r1, const RIB *r2);

public:
	rtObject();
	~rtObject();

	void	init();
	void	add_proto(rtProto* rp);
	int	command(int argc, const char*const* argv);
	void	intf_changed();
	void	compute_LocRIB();
	void	update_protos();
	void	dump_routes(Trace* t);

	RIB*	rts(int i)		{ return &rts_[i]; } 
	void	mark_routes_changed()	{ proto_rts_changed_ = 1; }
};

struct rtPeer {
	rtProto*	rtproto_;
	u_short		addr_, port_;	// locn. of peer
	Link*		if_;		// intf to reach peer
	RIB*		rtsin_;		// aka AdjRIBIn per peer
	char*		rtsout_;	// aka AdjRIBOut per peer
	rtPeer*		next_;
	u_short		state_;
	u_short		send_update_;
	short		pref_;		// default pref for peer
};
  
#define	DVInfinity	16
#define	DVRefreshTime	30
#define	DV_REFRESH_TIMER	 0

class rtProto : public Agent {
	char*		name_;		// of protocol
	int		pref_;		// global pref for proto
	rtPeer*		peers_;
	int		peer_rts_changed_;
	RIB*		rts_;		// consolidated AdjRIBIn per proto
	rtObject*	rtobj_;
	double		time_;		// of last update

	inline int	get_pref(const RIB* r1);
	inline int	BetterRoute(const RIB* r1, const int pref1,
				    const RIB*r2);
	inline rtPeer*	get_peer(int addr) {
		rtPeer* peer;
		for (peer = peers_; peer; peer = peer->next_)
			if (peer->addr_ == addr)
				break;
		return peer;
	}
	inline	int	addr()	{ return node_->addr(); }
	inline	int	port()	{ return sport_; } 
	void	peerinit(Link*, rtProto*);
		
public:
	rtProto*	next_;

			rtProto();
			~rtProto();
	char*		name()		{ return name_; } 
	double		time()		{ return time_; } 
	void		init();
	void		compute_RIBs();
	RIB*		rts(int i)	{ return &rts_[i]; } 
	inline void	notify_send_updates();
	inline void	send_update();
	void		send_update(rtPeer *p);

	void	recv(Packet *p);
	void	timeout(int tno);
	int	command(int argc, const char*const* argv);	
};  	

#endif /* ns_rtglib_h */

/*
 * Local Variables:
 * mode: c++
 * comment-column: 50
 * End:
 */

