/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * 	This product includes software developed by the Network Research
 * 	Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * PSC Changes Copyright (c) 1996 Pittsburgh Supercomputing Center,
 *                     Jamshid Mahdavi, Matt Mathis, and Jeff Semke
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the modifications to this
 * software were developed by the Pittsburgh Supercomputing Center.
 * The name of the Center may not be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE
 * IMPLIED WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE. 
 *
 */



#ifndef lint
static char rcsid[] =
    "@(#) $Header: scoreboard.cc,v 1.5 96/01/27 15:40:40 mccanne Exp $ (LBL)";
#endif

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <math.h>

#include "packet.h"
#include "scoreboardrh.h"
#include "tcp.h"

#define ASSERT(x) if (!(x)) {printf ("Assert SB failed\n"); exit(1);}
#define ASSERT1(x) if (!(x)) {printf ("Assert1 SB (length)\n"); exit(1);}

#define SBNI SBN[i%SBSIZE]

// last_ack = TCP last ack
int 
ScoreBoardRH::UpdateScoreBoard (int last_ack, int maxseq, Packet *pkt)
{
	int i, sack_index, sack_left, sack_right;
	int retran_decr = 0;

	//  If there is no scoreboard, create one.
	if (length_ == 0
	    && pkt->bd_.tcp_.sa_length_) {
		i = last_ack+1;
		SBNI.seq_no_ = i;
		SBNI.ack_flag_ = 0;
		SBNI.sack_flag_ = 0;
		SBNI.retran_ = 0;
		SBNI.snd_nxt_ = 0;
		SBNI.sack_cnt_ = 0;
		first_ = i%SBSIZE;
		length_++;
		if (length_ >= SBSIZE) {
			printf ("Error, scoreboard too large\n");
			exit(1);
		}
	}	
	//  Advance the left edge of the block.
	if (length_ && (SBN[first_].seq_no_ <= last_ack)) {
	      for (i=SBN[(first_)%SBSIZE].seq_no_; i<=last_ack; i++) {
		    //  Advance the ACK
		    if (SBNI.seq_no_ <= last_ack) {
			  ASSERT(first_ == i%SBSIZE);
			  first_ = (first_+1)%SBSIZE; 
			  length_--;
			  ASSERT1(length_ >= 0);
			  SBNI.ack_flag_ = 1;
			  SBNI.sack_flag_ = 1;
			  SBNI.sack_cnt_ = 0;
			  if (SBNI.retran_) {
				SBNI.retran_ = 0;
				SBNI.snd_nxt_ = 0;
				retran_decr++;
			  }
			  if (length_==0) 
				break;
		    }
	      }
	}

	bd_tcp* bd = &pkt->bd_.tcp_;
	for (sack_index=0; sack_index < bd->sa_length_; sack_index++) {
		sack_left = bd->sa_left_[sack_index];
		sack_right = bd->sa_right_[sack_index];
		
		if (sack_right - 1 > GetFack())
		      SetFack(sack_right - 1);

		//  Create new entries off the right side.
		if (sack_right > SBN[(first_+length_+SBSIZE-1)%SBSIZE].seq_no_) {
			//  Create new entries
			for (i = SBN[(first_+length_+SBSIZE-1)%SBSIZE].seq_no_+1; i<sack_right; i++) {
				SBNI.seq_no_ = i;
				SBNI.ack_flag_ = 0;
				SBNI.sack_flag_ = 0;
				SBNI.retran_ = 0;
				SBNI.snd_nxt_ = 0;
				SBNI.sack_cnt_ = 0;
				length_++;
				if (length_ >= SBSIZE) {
					printf ("Error, scoreboard too large\n");
					exit(1);
				}
			}
		}

		
		for (i=SBN[(first_)%SBSIZE].seq_no_; i<sack_right; i++) {
			//  Check to see if this segment is now covered by the sack block
			if (SBNI.seq_no_ >= sack_left && SBNI.seq_no_ < sack_right) {
			        SBNI.sack_cnt_ = 0;
				if (! SBNI.sack_flag_) {
					SBNI.sack_flag_ = 1;
				}
				if (SBNI.retran_) {
				  SBNI.retran_ = 0;
				  retran_decr++;
				}
			}
		}
	}
	/* Rescan the scoreboard and perform two updates:
	 * - Reset retransmitted ptr for all holes who's saved
	 *   snd_nxt_ is behind fack
	 * - Increment the "observed" count for all holes
	 *   to the left of right-most edge in this SACK block (bfack)
	 */
	for (i = SBN[(first_)%SBSIZE].seq_no_; 
	     i < SBN[(first_)%SBSIZE].seq_no_ + length_; i++) {
	      if ((SBNI.snd_nxt_ < GetFack()) && SBNI.retran_ ) {
		    // retransmitted pkt dropped again
		    retran_decr++;
		    SBNI.retran_ = 0;
		    SBNI.sack_cnt_ = 0;
		    SBNI.snd_nxt_ = maxseq;
	      }
	      else if (!SBNI.sack_flag_) {
		    SBNI.sack_cnt_++;
	      }
	}

	return (retran_decr);
}

/*
 * GetNextRetran() returns "-1" if there is no packet that is
 *   not acked and not sacked and not retransmitted.
 */
int 
ScoreBoardRH::GetNextRetran()	    // Returns sequence number of next pkt...
{
	int i;

	if (length_) {
	      for (i=SBN[(first_)%SBSIZE].seq_no_; 
		   i<SBN[(first_)%SBSIZE].seq_no_+length_; i++) {

		    if (!SBNI.ack_flag_ && !SBNI.sack_flag_ && !SBNI.retran_
			&& (SBNI.sack_cnt_ >= NUMDUPACKS)) {
			  return (i);
		    }
	      }
	}
	return (-1);
}


