#
# this is still somewhat experimental,
# and should be considered of 'alpha-test' quality  (kfall@ee.lbl.gov)
#
#
# Copyright (c) 1997 The Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#	This product includes software developed by the Computer Systems
#	Engineering Group at Lawrence Berkeley Laboratory.
# 4. Neither the name of the University nor of the Laboratory may be used
#    to endorse or promote products derived from this software without
#    specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# @(#) $Header: test-suite-fulltcp.tcl,v 1.6 97/05/28 17:17:58 kfall Exp $ (LBL)
#
# test-suite-fulltcp.tcl
#
# This file tests "fulltcp", a version of tcp reno implemented in the
# simulator based on the BSD Net/3 code.
#
# Create a simple four node topology:
#
#        s1
#         \ 
#  8Mb,5ms \  0.8Mb,100ms
#           r1 --------- k1
#  8Mb,5ms /
#         /
#        s2
#
proc create_testnet qlim {

	global s1 s2 r1 k1
	set s1 [ns node]
	set s2 [ns node]
	set r1 [ns node]
	set k1 [ns node]

	ns_duplex $s1 $r1 8Mb 10ms drop-tail
	ns_duplex $s2 $r1 8Mb 10ms drop-tail
	set L [ns_duplex $r1 $k1 800Kb 90ms drop-tail]
	[lindex $L 0] set queue-limit $qlim
	[lindex $L 1] set queue-limit $qlim
}

proc create_testnet1 { } {

	global s1 s2 r1 k1
	set s1 [ns node]
	set s2 [ns node]
	set r1 [ns node]
	set k1 [ns node]

	ns_duplex $s1 $r1 10Mb 5ms drop-tail
	ns_duplex $s2 $r1 10Mb 5ms drop-tail
	set L [ns_duplex $r1 $k1 1.5Mb 100ms drop-tail]
	[lindex $L 0] set queue-limit 23
	[lindex $L 1] set queue-limit 23
}

#
# Create a simple six node topology:
#
#        s1		    s3
#         \ 		    /
# 10Mb,2ms \  1.5Mb,20ms   / 10Mb,4ms
#           r1 --------- r2
# 10Mb,3ms /		   \ 10Mb,5ms
#         /		    \
#        s2		    s4 
#
proc create_testnet2 { } {

	global s1 s2 r1 r2 s3 s4
	set s1 [ns node]
	set s2 [ns node]
	set r1 [ns node]
	set r2 [ns node]
	set s3 [ns node]
	set s4 [ns node]

	ns_duplex $s1 $r1 10Mb 2ms drop-tail
	ns_duplex $s2 $r1 10Mb 3ms drop-tail
	set L [ns_duplex $r1 $r2 1.5Mb 20ms red]
	[lindex $L 0] set queue-limit 25
	[lindex $L 1] set queue-limit 25
	ns_duplex $s3 $r2 10Mb 4ms drop-tail
	ns_duplex $s4 $r2 10Mb 5ms drop-tail
}

proc finish { testname tr1 tr2 segfile ackfile } {

	global exit_scheduled

	$tr1 flush
	$tr2 flush

	set now [ns-now]
	if { ![info exists exit_scheduled] } {
		ns at [expr $now + 0.1] "exit 0"
		set exit_scheduled true
	}

	set fname [ns-random 0]
	set ackstmp $fname.acks ; # data-full acks
	set packstmp $fname.packs; # pure acks
	set segstmp $fname.p
	set dropstmp $fname.d
	set grtmp $fname.xgr

	exec rm -f $grtmp $ackstmp $segstmp $dropstmp $packstmp
	set f [open $grtmp w]
	puts $f "TitleText: $testname"
	puts $f "Device: Postscript"

	exec touch $ackstmp $segstmp $dropstmp

	#
	# $5: packet type
	# $6: packet length
	# $11: seq number
	# $13: ack number
	# $15: TCP header length
	#
	exec awk {
		{
			if (($1 == "+") && (($5 == "tcp") || ($5 == "ack")) \
				&& ($6 != $15))
				print $2, $11 + $6 - $15 
		}
	} $segfile > $segstmp
	# this part uses the "ackno" field of the tcp header (for fulltcp)
	#	ACKs with data
	exec awk {
		{
			if (($1 == "+") && (($5 == "tcp") || ($5 == "ack")) \
				&& ($6 != $15))
				print $2, $13
		}
	} $ackfile > $ackstmp
	#	pure ACKS
	exec awk {
		{
			if (($1 == "+") && (($5 == "tcp") || ($5 == "ack")) \
				&& ($6 == $15))
				print $2, $13
		}
	} $ackfile > $packstmp
	exec awk {
		{
			if ($1 == "d")
				print $2, $11 + $6 - $15
		}
	} $segfile > $dropstmp

	#
	# build seqno data set
	#
	puts $f \"seqno
	exec cat $segstmp >@ $f

	#
	# build acks dataset
	#
	puts $f [format "\n\"acks w/data"]
	exec cat $ackstmp >@ $f

	puts $f [format "\n\"pure acks"]
	exec cat $packstmp >@ $f

	#
	# build drops dataset
	#
	puts $f [format "\n\"drops\n"]
	#
	# Repeat the first line twice in the drops file because
	# often we have only one drop and xgraph won't print marks
	# for data sets with only one point.
	#
	exec head -1 $dropstmp >@ $f
	exec cat $dropstmp >@ $f
	flush $f
	close $f
	exec xgraph -bb -nl -m -x time -y seqno $grtmp &
	exec /bin/sleep 1
	exec rm -f $grtmp $ackstmp $segstmp $dropstmp $packstmp
}

proc openTrace fname {
	exec rm -f $fname
	set traceFile [open $fname w]
	set T [ns trace]
	$T attach $traceFile
	$T set show_tcphdr true
	return $T
}

#
# make this tcp look awfully close to
# a real BSD/reno tcp
#
proc bsdcompat tcp {
	$tcp set segsperack 2
	$tcp set dupseg_fix false
	$tcp set dupack_reset true
	$tcp set bug-fix false
	$tcp set data_on_syn false
}

proc test_full {} {
	global s1 s2 r1 k1
	create_testnet 16
	set stopt 6.0

	# the next part replaces call to 'ns_create_connection'
	#
	set src [ns agent tcp-full-reno $s1]
	set sink [ns agent tcp-full-reno $k1]
	$src set class 0
	$sink set class 0
	$src connect [$sink addr] [$sink port]
	$sink listen	; # will figure out who its peer is
	$src set window 100
	set ftp1 [$src source ftp]
	ns at 0.0 "$ftp1 start"

	set trace1 [openTrace out.tr]
	[ns link $r1 $k1] trace $trace1

	set trace2 [openTrace acks.tr]
	[ns link $r1 $s1] trace $trace2

	ns at $stopt "finish test_full $trace1 $trace2 out.tr acks.tr"
	ns run
}

proc test_close {} {
	global s1 s2 r1 k1
	create_testnet 16
	set stopt 6.0

	# the next part replaces call to 'ns_create_connection'
	#
	set src [ns agent tcp-full-reno $s1]
	set sink [ns agent tcp-full-reno $k1]
	$src set class 0
	$sink set class 0
	$src connect [$sink addr] [$sink port]
	$sink listen	; # will figure out who its peer is
	$src set window 100
	set ftp1 [$src source ftp]
	ns at 0.0 "$ftp1 produce 50"
	ns at 5.5 "$src close"

	set trace1 [openTrace out.tr]
	[ns link $r1 $k1] trace $trace1

	set trace2 [openTrace acks.tr]
	[ns link $r1 $s1] trace $trace2

	ns at $stopt "finish test_close $trace1 $trace2 out.tr acks.tr"
	ns run
}

proc test_twoway {} {
	global s1 s2 r1 k1
	create_testnet 16
	set stopt 6.0
	set startt 3.0

	# the next part replaces call to 'ns_create_connection'
	#
	set src [ns agent tcp-full-reno $s1]
	set sink [ns agent tcp-full-reno $k1]
	$src set class 0
	$sink set class 0
	$src connect [$sink addr] [$sink port]
	$sink listen	; # will figure out who its peer is
	$sink set iss 2144
	$src set window 100
	set ftp1 [$src source ftp]
	ns at 0.0 "$ftp1 start"
	set ftp2 [$sink source ftp]
	ns at $startt "$ftp2 start"

	set trace1 [openTrace out.tr]
	[ns link $r1 $k1] trace $trace1

	set trace2 [openTrace acks.tr]
	[ns link $r1 $s1] trace $trace2

	ns at $stopt "finish test_twoway_forward $trace1 $trace2 out.tr acks.tr"
	ns run
}
proc test_twoway0 {} {
	global s1 s2 r1 k1
	create_testnet 16
	set stopt 6.0
	set startt 0.0

	# the next part replaces call to 'ns_create_connection'
	#
	set src [ns agent tcp-full-reno $s1]
	set sink [ns agent tcp-full-reno $k1]
	$src set class 0
	$sink set class 0
	$src connect [$sink addr] [$sink port]
	$sink listen	; # will figure out who its peer is
	$sink set iss 2144
	$src set window 100
	$sink set window 100
	set ftp1 [$src source ftp]
	ns at 0.0 "$ftp1 start"
	set ftp2 [$sink source ftp]
	ns at $startt "$ftp2 start"

	set trace1 [openTrace out.tr]
	[ns link $r1 $k1] trace $trace1

	set trace2 [openTrace acks.tr]
	[ns link $r1 $s1] trace $trace2

	ns at $stopt "finish test_twoway0_forward $trace1 $trace2 out.tr acks.tr"
	ns at $stopt "finish test_twoway0_reverse $trace1 $trace2 acks.tr out.tr"
	ns run
}

proc test_twoway1 {} {
	global s1 s2 r1 k1
	create_testnet 16
	set stopt 6.0
	set startt 1.9

	# the next part replaces call to 'ns_create_connection'
	#
	set src [ns agent tcp-full-reno $s1]
	set sink [ns agent tcp-full-reno $k1]
	$src set class 0
	$sink set class 0
	$src connect [$sink addr] [$sink port]
	$sink listen	; # will figure out who its peer is
	$sink set iss 2144
	$src set window 100
	$sink set window 100
	set ftp1 [$src source ftp]
	ns at 0.0 "$ftp1 start"
	set ftp2 [$sink source ftp]
	ns at $startt "$ftp2 start"

	set trace1 [openTrace out.tr]
	[ns link $r1 $k1] trace $trace1

	set trace2 [openTrace acks.tr]
	[ns link $r1 $s1] trace $trace2

	ns at $stopt "finish test_twoway1_forward $trace1 $trace2 out.tr acks.tr"
	ns at $stopt "finish test_twoway1_reverse $trace1 $trace2 acks.tr out.tr"
	ns run
}

proc test_twoway_bsdcompat {} {
	global s1 s2 r1 k1
	create_testnet 16
	set stopt 6.0
	set startt 1.9

	# the next part replaces call to 'ns_create_connection'
	#
	set src [ns agent tcp-full-reno $s1]
	set sink [ns agent tcp-full-reno $k1]
	$src set class 0
	$sink set class 0
	$src connect [$sink addr] [$sink port]
	$sink listen	; # will figure out who its peer is
	$src set window 100
	$sink set window 100
	bsdcompat $src
	bsdcompat $sink
	set ftp1 [$src source ftp]
	ns at 0.0 "$ftp1 start"
	set ftp2 [$sink source ftp]
	ns at $startt "$ftp2 start"

	set trace1 [openTrace out.tr]
	[ns link $r1 $k1] trace $trace1

	set trace2 [openTrace acks.tr]
	[ns link $r1 $s1] trace $trace2

	ns at $stopt "finish bsdcompat_twoway1_forward $trace1 $trace2 out.tr acks.tr"
	ns at $stopt "finish bsdcompat_twoway1_reverse $trace1 $trace2 acks.tr out.tr"
	ns run
}

proc test_oneway_bsdcompat {} {
	global s1 s2 r1 k1
	create_testnet 16
	set stopt 6.0
	set startt 1.9

	# the next part replaces call to 'ns_create_connection'
	#
	set src [ns agent tcp-full-reno $s1]
	set sink [ns agent tcp-full-reno $k1]
	$src set class 0
	$sink set class 0
	$src connect [$sink addr] [$sink port]
	$sink listen	; # will figure out who its peer is
	$src set window 100
	$sink set window 100
	bsdcompat $src
	bsdcompat $sink
	set ftp1 [$src source ftp]
	ns at 0.0 "$ftp1 start"

	set trace1 [openTrace out.tr]
	[ns link $r1 $k1] trace $trace1

	set trace2 [openTrace acks.tr]
	[ns link $r1 $s1] trace $trace2

	ns at $stopt "finish bsdcompat_oneway $trace1 $trace2 out.tr acks.tr"
	ns run
}

proc test_twowayrandom {} {
	global s1 s2 r1 k1
	create_testnet 16
	set stopt 6.0
	set startt [expr [ns-random 0] % 6]
	puts "second TCP starting at time $startt"

	# the next part replaces call to 'ns_create_connection'
	#
	set src [ns agent tcp-full-reno $s1]
	set sink [ns agent tcp-full-reno $k1]
	$src set class 0
	$sink set class 0
	$src connect [$sink addr] [$sink port]
	$sink listen	; # will figure out who its peer is
	$sink set iss 2144
	$src set window 100
	$sink set window 100
	set ftp1 [$src source ftp]
	ns at 0.0 "$ftp1 start"
	set ftp2 [$sink source ftp]
	ns at $startt "$ftp2 start"

	set trace1 [openTrace out.tr]
	[ns link $r1 $k1] trace $trace1

	set trace2 [openTrace acks.tr]
	[ns link $r1 $s1] trace $trace2

	ns at $stopt "finish test_twowayrandom $trace1 $trace2 out.tr acks.tr"
	ns run
}

proc test_delack {} {
	global s1 s2 r1 k1
	create_testnet 16
	set stopt 6.0

	# the next part replaces call to 'ns_create_connection'
	#
	set src [ns agent tcp-full-reno $s1]
	set sink [ns agent tcp-full-reno $k1]
	$src set class 0
	$sink set class 0
	$src connect [$sink addr] [$sink port]
	$sink listen	; # will figure out who its peer is
	$sink set segsperack 2
	$src set window 100
	set ftp1 [$src source ftp]
	ns at 0.0 "$ftp1 start"

	set trace1 [openTrace out.tr]
	[ns link $r1 $k1] trace $trace1

	set trace2 [openTrace acks.tr]
	[ns link $r1 $s1] trace $trace2

	ns at $stopt "finish test_delack $trace1 $trace2 out.tr acks.tr"
	ns run
}

if { $argc != 1 } {
	puts stderr {usage: ns test-suite-fulltcp.tcl [full twoway ...] }
	exit 1
}
if { "[info procs test_$argv]" != "test_$argv" } {
	puts stderr "test-suite-fulltcp.tcl: no such test: $argv"
}
test_$argv
