#
# Copyright (c) 1995 The Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#	This product includes software developed by the Computer Systems
#	Engineering Group at Lawrence Berkeley Laboratory.
# 4. Neither the name of the University nor of the Laboratory may be used
#    to endorse or promote products derived from this software without
#    specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# @(#) $Header: /nfs/prot/kannan/PhD/simulators/ns/ns-1.0+rtg/RCS/test-suite.tcl,v 1.1 1996/08/09 17:10:41 kannan Exp kannan $
#

#
# These are assorted support routines for running any ns routine.
#

#
# Arrange for tcp source stats to be dumped for $tcpSrc every
# $interval seconds of simulation time
#
proc tcpDump { tcpSrc interval } {
	proc dump { src interval } {
		ns at [expr [ns now] + $interval] "dump $src $interval"
		puts [ns now]/cwnd=[$src get cwnd]/ssthresh=[$src get ssthresh]/ack=[$src get ack]
	}
	ns at 0.0 "dump $tcpSrc $interval"
}

proc tcpDumpAll { tcpSrc interval label } {
	proc dump { src interval label } {
		ns at [expr [ns now] + $interval] "dump $src $interval $label"
		puts $label/time=[ns now]/cwnd=[$src get cwnd]/ssthresh=[$src get ssthresh]/ack=[$src get ack]/rtt=[$src get rtt]
	}
	puts $label:window=[$tcpSrc get window]/packet-size=[$tcpSrc get packet-size]/bug-fix=[$tcpSrc get bug-fix]
	ns at 0.0 "dump $tcpSrc $interval $label"
}


#
# file operations
#

proc openTrace { stopTime testName } {
	exec rm -f out.tr temp.rands
	set traceFile [open out.tr w]
	ns at $stopTime \
		"close $traceFile ; finish $testName"
	set T [ns trace]
	$T attach $traceFile
	return $T
}

proc finish file {

	set f [open temp.rands w]
	puts $f "TitleText: $file"
	puts $f "Device: Postscript"
	
	exec rm -f temp.p temp.d temp.f
	exec touch temp.d temp.p temp.f
	#
	# split queue/drop events into two separate files.
	# we don't bother checking for the link we're interested in
	# since we know only such events are in our trace file
	# If the link were dynamic, gather a third file.
	#
	exec awk {
	    BEGIN { print "\"packets" }
	    {
		if (($1 == "+" || $1 == "-" ) && \
			($5 == "tcp" || $5 == "ack"))\
			print $2, $8 + ($11 % 90) * 0.01
	    }
	} out.tr > temp.p
	exec awk {
	    BEGIN {
		print ""
		print "\"drops" 
		first = 1
		lf = 0
	    }
	    /link-fail/ {
		stats[$2] = $8 + ($11 % 90) * 0.01
		lf = 1
		next;
	    }
	    {
		if ($1 == "d") {
		    print $2, $8 + ($11 % 90) * 0.01
		    if (first == 1) {
			#
			# Repeat the first line twice in the drops file
			# because often we have only one drop and
			# xgraph won't print marks for data sets
			# with only one point.
			#
			print $2, $8 + ($11 % 90) * 0.01
			first = 0
		    }
		}
	    }
	    END {
		if (lf == 1) {
		    print ""
		    print "\"rt-fail drops"
		    for (time in stats) {
			print time, stats[time]
		    }
		}
	    }
	} out.tr > temp.d
	exec awk {
	    BEGIN {
		ld = 0
		iv = 0
	    }
	    /^U/ {
		ld = 1
		up[$2] = 1
		next;
	    }
	    /^D/ {
		dn[$2] = 1;
		next;
	    }
	    {
		if ($8 > iv) iv = $8
	    }
	    END {
		if (ld == 1) {
		    iv++
		    print ""
		    print "\"link-up"
		    for (time in up) {
			for (i = 0; i <= iv; i++) {
			    print time, i
			    print time, i
			}
		    }
		    print ""
		    print "\"link-down"
		    for (time in dn) {
			for (i = 0; i <= iv; i++) {
			    print time, i
			    print time, i
			}
		    }
		}
	    }
	} out.tr > temp.f

	exec cat temp.p >@ $f		  ;# packets
	flush $f

	if {[file size temp.f] > 0} {
	    exec cat temp.f >@ $f
	    flush $f
	} else {
	    # insert dummy data sets so we get X's for marks in data-set 4
	    puts $f [format "\n\"skip-1\n0 1\n\n\"skip-2\n0 1\n"]
	}

	exec cat temp.d >@ $f		  ;# drops
	close $f
	exec xgraph -bb -tk -nl -m -x time -y packet temp.rands &
	
	exit 0
}

proc error { msg } {
    puts stderr "test-suite.tcl: $msg"
}

