#
# Copyright (c) 1995 The Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#	This product includes software developed by the Computer Systems
#	Engineering Group at Lawrence Berkeley Laboratory.
# 4. Neither the name of the University nor of the Laboratory may be used
#    to endorse or promote products derived from this software without
#    specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# @(#) $Header: /nfs/prot/kannan/PhD/simulators/ns/ns-1.0+rtg/RCS/test-suite.tcl,v 1.1 1996/08/09 17:10:41 kannan Exp kannan $
#
#
# This test suite reproduces most of the tests from the following note:
# Floyd, S., Simulator Tests. July 1995.  
# URL ftp://ftp.ee.lbl.gov/papers/simtests.ps.Z.
#
# To run all tests:  test-all
# To run individual tests:
# ns test-suite.tcl tahoe1
# ns test-suite.tcl tahoe2
# ...

proc test_tahoe1 {} {
	global s1 s2 r1 k1
	create_testnet
	set tcp1 [ns_create_connection tcp $s1 tcp-sink $k1 0]
	$tcp1 set window 50
	set ftp1 [$tcp1 source ftp]
	ns at 0.0 "$ftp1 start"

	tcpDump $tcp1 1.0

	# trace only the bottleneck link
	[ns link $r1 $k1] trace [openTrace 5.0 test_tahoe]
}

proc test_tahoe2 {} {
	global s1 s2 r1 k1
	create_testnet

	puts stderr "ns: run test tahoe2"
	set tcp1 [ns_create_connection tcp $s1 tcp-sink $k1 0]
	$tcp1 set window 14
	set ftp1 [$tcp1 source ftp]

	ns at 1.0 "$ftp1 start"

	tcpDump $tcp1 1.0

	# trace only the bottleneck link
	[ns link $r1 $k1] trace [openTrace 5.0 test_tahoe2]
}

proc test_tahoe3 {} {
	global s1 s2 r1 k1
	create_testnet
	[ns link $r1 $k1] set queue-limit 8
	[ns link $k1 $r1] set queue-limit 8
	
	set tcp1 [ns_create_connection tcp $s1 tcp-sink $k1 0]
	$tcp1 set window 100

	set tcp2 [ns_create_connection tcp $s2 tcp-sink $k1 1]
	$tcp2 set window 16

	set ftp1 [$tcp1 source ftp]
	set ftp2 [$tcp2 source ftp]

	ns at 1.0 "$ftp1 start"
	ns at 0.5 "$ftp2 start"

	tcpDump $tcp1 1.0

	# trace only the bottleneck link
	[ns link $r1 $k1] trace [openTrace 8.0 test_tahoe3]
}

proc test_tahoe4 {} {
	global s1 s2 r1 k1
	create_testnet
	[ns link $s2 $r1] set delay 200ms
	[ns link $r1 $s2] set delay 200ms
	[ns link $r1 $k1] set queue-limit 11
	[ns link $k1 $r1] set queue-limit 11
	
	set tcp1 [ns_create_connection tcp $s1 tcp-sink $k1 0]
	$tcp1 set window 30

	set tcp2 [ns_create_connection tcp $s2 tcp-sink $k1 1]
	$tcp2 set window 30

	set ftp1 [$tcp1 source ftp]
	set ftp2 [$tcp2 source ftp]

	ns at 0.0 "$ftp1 start"
	ns at 0.0 "$ftp2 start"

	tcpDump $tcp1 5.0

	# trace only the bottleneck link
	[ns link $r1 $k1] trace [openTrace 25.0 test_tahoe4]
}

# No tahoe5.  We want to keep the tahoe and reno analogues.

proc test_tahoe6 {} {
    # tahoe1 setup for dynamic routed scenarios
    global s1 s2 r1 r2 k1
    create_testnet3
    set tcp1 [ns_create_connection tcp $s1 tcp-sink $k1 0]
    $tcp1 set window 50
    set ftp1 [$tcp1 source ftp]
    ns at 0.0 "$ftp1 start"

    tcpDump $tcp1 1.0

    # trace the bottleneck links
    set Trace [openTrace 5.0 test_tahoe6]
    [ns link $r1 $k1] trace $Trace
    [ns link $r1 $r2] trace $Trace
}

proc test_tahoe7 {} {
    # tahoe2 for dynamic routed scenarios
    global s1 s2 r1 r2 k1
    create_testnet3

    set tcp1 [ns_create_connection tcp $s1 tcp-sink $k1 0]
    $tcp1 set window 14
    set ftp1 [$tcp1 source ftp]

    ns at 1.0 "$ftp1 start"

    tcpDump $tcp1 1.0

    # trace the bottleneck links
    set Trace [openTrace 5.0 test_tahoe7]
    [ns link $r1 $k1] trace $Trace
    [ns link $r1 $r2] trace $Trace
}

proc test_tahoe8 {} {
    # tahoe3 for dynamic routed scenarios
    global s1 s2 r1 r2 k1
    create_testnet3
    [ns link $r1 $k1] set queue-limit 8
    [ns link $k1 $r1] set queue-limit 8
	
    set tcp1 [ns_create_connection tcp $s1 tcp-sink $k1 0]
    $tcp1 set window 100

    set tcp2 [ns_create_connection tcp $s2 tcp-sink $k1 1]
    $tcp2 set window 16

    set ftp1 [$tcp1 source ftp]
    set ftp2 [$tcp2 source ftp]

    ns at 1.0 "$ftp1 start"
    ns at 0.5 "$ftp2 start"

    tcpDump $tcp1 1.0

    # trace the bottleneck links
    set Trace [openTrace 8.0 test_tahoe8]
    [ns link $r1 $k1] trace $Trace
    [ns link $r1 $r2] trace $Trace
}

proc test_no_bug {} {

	global s1 s2 r1 k1
	create_testnet1
	[ns link $s1 $r1] set delay 3ms
	[ns link $r1 $s1] set delay 3ms 

	set tcp1 [ns_create_connection tcp $s1 tcp-sink $k1 0]
	$tcp1 set window 50

	set tcp2 [ns_create_connection tcp $s2 tcp-sink $k1 1]
	$tcp2 set window 50

	set ftp1 [$tcp1 source ftp]
	set ftp2 [$tcp2 source ftp]

	ns at 1.0 "$ftp1 start"
	ns at 1.75 "$ftp2 produce 100"

	tcpDump $tcp1 1.0

	# trace only the bottleneck link
	[ns link $r1 $k1] trace [openTrace 6.0 test_no_bug]
}

proc test_bug {} {

	global s1 s2 r1 k1
	create_testnet1
	[ns link $s1 $r1] set delay 3ms
	[ns link $r1 $s1] set delay 3ms 

	set tcp1 [ns_create_connection tcp $s1 tcp-sink $k1 0]
	$tcp1 set window 50
	$tcp1 set bug-fix false

	set tcp2 [ns_create_connection tcp $s2 tcp-sink $k1 1]
	$tcp2 set window 50
	$tcp2 set bug-fix false

	set ftp1 [$tcp1 source ftp]
	set ftp2 [$tcp2 source ftp]

	ns at 1.0 "$ftp1 start"
	ns at 1.75 "$ftp2 produce 100"

	tcpDump $tcp1 1.0

	# trace only the bottleneck link
	[ns link $r1 $k1] trace [openTrace 6.0 test_bug]
}

proc test_reno1 {} {
	global s1 s2 r1 k1
	create_testnet
	set tcp1 [ns_create_connection tcp-reno $s1 tcp-sink $k1 0]
	$tcp1 set window 14

	set ftp1 [$tcp1 source ftp]
	ns at 1.0 "$ftp1 start"

	tcpDump $tcp1 1.0

	# trace only the bottleneck link
	[ns link $r1 $k1] trace [openTrace 5.0 test_reno1]
}

proc test_reno {} {
	global s1 s2 r1 k1
	create_testnet
	set tcp1 [ns_create_connection tcp-reno $s1 tcp-sink $k1 0]
	$tcp1 set window 28
	$tcp1 set maxcwnd 14

	set ftp1 [$tcp1 source ftp]
	ns at 1.0 "$ftp1 start"

	tcpDump $tcp1 1.0

	# trace only the bottleneck link
	[ns link $r1 $k1] trace [openTrace 5.0 test_reno]
}

proc test_renoA {} {
	global s1 s2 r1 k1
	create_testnet
	[ns link $r1 $k1] set queue-limit 8
	set tcp1 [ns_create_connection tcp-reno $s1 tcp-sink $k1 0]
	$tcp1 set window 28
	set tcp2 [ns_create_connection tcp-reno $s1 tcp-sink $k1 1]
	$tcp2 set window 4
	set tcp3 [ns_create_connection tcp-reno $s1 tcp-sink $k1 2]
	$tcp3 set window 4

	set ftp1 [$tcp1 source ftp]
	ns at 1.0 "$ftp1 start"
	set ftp2 [$tcp2 source ftp]
	ns at 1.2 "$ftp2 start"
	$ftp2 set maxpkts 7
	set ftp3 [$tcp3 source ftp]
	ns at 1.2 "$ftp3 start"
	$ftp3 set maxpkts 7

	tcpDump $tcp1 1.0
	tcpDump $tcp2 1.0
	tcpDump $tcp3 1.0

	# trace only the bottleneck link
	[ns link $r1 $k1] trace [openTrace 5.0 test_renoA]
}

proc test_reno2 {} {
	global s1 s2 r1 k1
	create_testnet
	[ns link $r1 $k1] set queue-limit 9

	set tcp1 [ns_create_connection tcp-reno $s1 tcp-sink $k1 0]
	$tcp1 set window 50

	set tcp2 [ns_create_connection tcp-reno $s2 tcp-sink $k1 1]
	$tcp2 set window 20

	set ftp1 [$tcp1 source ftp]
	set ftp2 [$tcp2 source ftp]

	ns at 1.0 "$ftp1 start"
	ns at 1.0 "$ftp2 start"

	tcpDump $tcp1 1.0

	# trace only the bottleneck link
	[ns link $r1 $k1] trace [openTrace 10.0 test_reno2]
}

proc test_reno3 {} {
	global s1 s2 r1 k1
	create_testnet
	[ns link $r1 $k1] set queue-limit 8
	[ns link $k1 $r1] set queue-limit 8

	set tcp1 [ns_create_connection tcp-reno $s1 tcp-sink $k1 0]
	$tcp1 set window 100

	set tcp2 [ns_create_connection tcp-reno $s2 tcp-sink $k1 1]
	$tcp2 set window 16

	set ftp1 [$tcp1 source ftp]
	set ftp2 [$tcp2 source ftp]

	ns at 1.0 "$ftp1 start"
	ns at 0.5 "$ftp2 start"

	tcpDump $tcp1 1.0

	# trace only the bottleneck link
	[ns link $r1 $k1] trace [openTrace 8.0 test_reno3]
}

proc test_reno4 {} { 
        global s1 s2 r1 r2 s3 s4
        create_testnet2
        [ns link $r1 $r2] set queue-limit 29
        set tcp1 [ns_create_connection tcp-reno $s1 tcp-sink-da $r2 0]
        $tcp1 set window 80
 	$tcp1 set maxcwnd 40

        set ftp1 [$tcp1 source ftp]
        ns at 0.0 "$ftp1 start"

        tcpDump $tcp1 1.0

        # trace only the bottleneck link
        [ns link $s1 $r1] trace [openTrace 2.0 test_reno4]
}

proc test_reno4a {} { 
        global s1 s2 r1 r2 s3 s4
        create_testnet2
        [ns link $r1 $r2] set queue-limit 29
        set tcp1 [ns_create_connection tcp-reno $s1 tcp-sink-da $r2 0]
        $tcp1 set window 40
	$tcp1 set maxcwnd 40

        set ftp1 [$tcp1 source ftp]
        ns at 0.0 "$ftp1 start"

        tcpDump $tcp1 1.0

        # trace only the bottleneck link
        [ns link $s1 $r1] trace [openTrace 2.0 test_reno4]
}

proc test_reno5 {} {
        global s1 s2 r1 k1
        create_testnet
        [ns link $r1 $k1] set queue-limit 9

        set tcp1 [ns_create_connection tcp-reno $s1 tcp-sink $k1 0]
        $tcp1 set window 50
        $tcp1 set bug-fix false

        set tcp2 [ns_create_connection tcp-reno $s2 tcp-sink $k1 1]
        $tcp2 set window 20
        $tcp2 set bug-fix false

        set ftp1 [$tcp1 source ftp]
        set ftp2 [$tcp2 source ftp]

        ns at 1.0 "$ftp1 start"
        ns at 1.0 "$ftp2 start"

        tcpDump $tcp1 1.0

        # trace only the bottleneck link
        [ns link $r1 $k1] trace [openTrace 10.0 test_reno5]
}

proc test_reno6 {} {
    # testbed reno adapted for dynamics
    global s1 s2 r1 r2 k1
    create_testnet3
    set tcp1 [ns_create_connection tcp-reno $s1 tcp-sink $k1 0]
    $tcp1 set window 28
    $tcp1 set maxcwnd 14

    set ftp1 [$tcp1 source ftp]
    ns at 1.0 "$ftp1 start"

    tcpDump $tcp1 1.0

    # trace the bottleneck links
    set Trace [openTrace 5.0 test_reno6]
    [ns link $r1 $k1] trace $Trace
    [ns link $r1 $r2] trace $Trace
}

proc test_reno7 {} {
    # testbed reno1, parameters analogous to tahoe7 and tahoe2
    global s1 s2 r1 r2 k1
    create_testnet3
    set tcp1 [ns_create_connection tcp-reno $s1 tcp-sink $k1 0]
    $tcp1 set window 14

    set ftp1 [$tcp1 source ftp]
    ns at 1.0 "$ftp1 start"

    tcpDump $tcp1 1.0

    # trace the bottleneck links
    set Trace [openTrace 5.0 test_reno7]
    [ns link $r1 $k1] trace $Trace
    [ns link $r1 $r2] trace $Trace
}

proc test_reno8 {} {
    # testbed reno3, analogoues to tahoe8 and tahoe3
    global s1 s2 r1 r2 k1
    create_testnet3
    [ns link $r1 $k1] set queue-limit 8
    [ns link $k1 $r1] set queue-limit 8

    set tcp1 [ns_create_connection tcp-reno $s1 tcp-sink $k1 0]
    $tcp1 set window 100

    set tcp2 [ns_create_connection tcp-reno $s2 tcp-sink $k1 1]
    $tcp2 set window 16

    set ftp1 [$tcp1 source ftp]
    set ftp2 [$tcp2 source ftp]

    ns at 1.0 "$ftp1 start"
    ns at 0.5 "$ftp2 start"

    tcpDump $tcp1 1.0

    # trace the bottleneck links
    set Trace [openTrace 8.0 test_reno8]
    [ns link $r1 $k1] trace $Trace
    [ns link $r1 $r2] trace $Trace
}


# Note that the telnet test includes random source behavior - its output
# is not currently repeatable from run to run
proc test_telnet {} {
	global s1 s2 r1 k1
	create_testnet
	[ns link $r1 $k1] set queue-limit 8
	[ns link $k1 $r1] set queue-limit 8

	set tcp1 [ns_create_connection tcp-reno $s1 tcp-sink $k1 0]
	set tcp2 [ns_create_connection tcp-reno $s2 tcp-sink $k1 1]
	set tcp3 [ns_create_connection tcp-reno $s2 tcp-sink $k1 2]

	set telnet1 [$tcp1 source telnet] ; $telnet1 set interval 1
	set telnet2 [$tcp2 source telnet] ; $telnet2 set interval 0
 	# Interval 0 designates the tcplib telnet interarrival distribution
	set telnet3 [$tcp3 source telnet] ; $telnet3 set interval 0

	ns at 0.0 "$telnet1 start"
	ns at 0.0 "$telnet2 start"
	ns at 0.0 "$telnet3 start"

	tcpDump $tcp1 5.0

	# trace only the bottleneck link
	[ns link $r1 $k1] trace [openTrace 50.0 test_telnet]

	# use a different seed each time
	puts seed=[ns random 0]
	error "Note that the telnet test includes random source behaviour."
	error "Its output is not currently repeatable from run to run."
}

proc test_delayed {} {
	global s1 s2 r1 k1
	create_testnet
	set tcp1 [ns_create_connection tcp $s1 tcp-sink-da $k1 0]
	$tcp1 set window 50

	# lookup up the sink and set it's delay interval
	[$k1 agent [$tcp1 dst-port]] set interval 100ms

	set ftp1 [$tcp1 source ftp]
	ns at 1.0 "$ftp1 start"

	tcpDump $tcp1 1.0

	# trace only the bottleneck link
	[ns link $r1 $k1] trace [openTrace 4.0 test_delayed]
}

proc test_phase {} {
	global s1 s2 r1 k1
	create_testnet
	[ns link $s2 $r1] set delay 3ms
	[ns link $r1 $s2] set delay 3ms
	[ns link $r1 $k1] set queue-limit 16
	[ns link $k1 $r1] set queue-limit 100

	set tcp1 [ns_create_connection tcp $s1 tcp-sink $k1 0]
	$tcp1 set window 32

	set tcp2 [ns_create_connection tcp $s2 tcp-sink $k1 1]
	$tcp2 set window 32

	set ftp1 [$tcp1 source ftp]
	set ftp2 [$tcp2 source ftp]

	ns at 5.0 "$ftp1 start"
	ns at 1.0 "$ftp2 start"

	tcpDump $tcp1 5.0

	# trace only the bottleneck link
	[ns link $r1 $k1] trace [openTrace 25.0 test_phase]
}

proc test_phase1 {} {
	global s1 s2 r1 k1
	create_testnet
	[ns link $s2 $r1] set delay 9.5ms
	[ns link $r1 $s2] set delay 9.5ms
	[ns link $r1 $k1] set queue-limit 16
	[ns link $k1 $r1] set queue-limit 100

	set tcp1 [ns_create_connection tcp $s1 tcp-sink $k1 0]
	$tcp1 set window 32

	set tcp2 [ns_create_connection tcp $s2 tcp-sink $k1 1]
	$tcp2 set window 32

	set ftp1 [$tcp1 source ftp]
	set ftp2 [$tcp2 source ftp]

	ns at 5.0 "$ftp1 start"
	ns at 1.0 "$ftp2 start"

	tcpDump $tcp1 5.0

	# trace only the bottleneck link
	[ns link $r1 $k1] trace [openTrace 25.0 test_phase1]
}

proc test_phase2 {} {
        global s1 s2 r1 k1
        create_testnet  
        [ns link $s2 $r1] set delay 3ms
        [ns link $r1 $s2] set delay 3ms
        [ns link $r1 $k1] set queue-limit 16
        [ns link $k1 $r1] set queue-limit 100

        set tcp1 [ns_create_connection tcp $s1 tcp-sink $k1 0]
        $tcp1 set window 32
        $tcp1 set overhead 0.01

        set tcp2 [ns_create_connection tcp $s2 tcp-sink $k1 1]
        $tcp2 set window 32
        $tcp2 set overhead 0.01
        
        set ftp1 [$tcp1 source ftp]
        set ftp2 [$tcp2 source ftp]
 
        ns at 5.0 "$ftp1 start"
        ns at 1.0 "$ftp2 start"
        
        tcpDump $tcp1 5.0
 
        # trace only the bottleneck link
        [ns link $r1 $k1] trace [openTrace 25.0 test_phase2]
}

proc test_timers {} {
	global s1 s2 r1 k1
	create_testnet
	[ns link $r1 $k1] set queue-limit 2
	[ns link $k1 $r1] set queue-limit 100

	set tcp1 [ns_create_connection tcp $s1 tcp-sink-da $k1 0]
	$tcp1 set window 4
	# lookup up the sink and set it's delay interval
	[$k1 agent [$tcp1 dst-port]] set interval 100ms

	set tcp2 [ns_create_connection tcp $s2 tcp-sink-da $k1 1]
	$tcp2 set window 4
	# lookup up the sink and set it's delay interval
	[$k1 agent [$tcp2 dst-port]] set interval 100ms

	set ftp1 [$tcp1 source ftp]
	set ftp2 [$tcp2 source ftp]

	ns at 1.0 "$ftp1 start"
	ns at 1.3225 "$ftp2 start"

	tcpDump $tcp1 5.0

	# trace only the bottleneck link
	[ns link $r1 $k1] trace [openTrace 10.0 test_timers]
}

proc test_stats {} {
	global s1 s2 r1 k1
	create_testnet
	[ns link $s2 $r1] set delay 200ms
	[ns link $r1 $s2] set delay 200ms
	[ns link $r1 $k1] set queue-limit 10
	[ns link $k1 $r1] set queue-limit 10
	
	set tcp1 [ns_create_connection tcp $s1 tcp-sink $k1 0]
	$tcp1 set window 30

	set tcp2 [ns_create_connection tcp $s2 tcp-sink $k1 1]
	$tcp2 set window 30

	set ftp1 [$tcp1 source ftp]
	set ftp2 [$tcp2 source ftp]

	ns at 1.0 "$ftp1 start"
	ns at 1.0 "$ftp2 start"

	tcpDumpAll $tcp1 5.0 tcp1
	tcpDumpAll $tcp2 5.0 tcp2

	# trace only the bottleneck link
	[ns link $r1 $k1] trace [openTrace 10.0 test_stats]
}
